/*
 * Copyright 2005, Elixir Technology Pte Ltd
 * 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer. 
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * Neither the name of the Elixir Technology Pte Ltd nor the names of its
 * contributors may be used to endorse or promote products derived from this
 * software without specific prior written permission.
 * 
 * This source code is provided as  a reference implementation. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL Elixir Technology Pte Ltd, 
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
// ViewerToolBar.java

package demo;

import java.awt.*;
import java.awt.event.*;
import java.io.InputStream;
import java.util.Properties;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import com.elixirtech.ui.JNavigationPanel;
import com.elixirtech.ui.INavigation;
import com.elixirtech.ui.images.Icons;
import com.elixirtech.report2.physical.viewer.IViewerCanvas;
import com.elixirtech.report2.physical.viewer.Zoomable;
import com.elixirtech.report2.print.IPrinter;
import com.elixirtech.report2.print.Printable;
import com.elixirtech.report2.print.PrinterUtil;

public class ViewerToolBar extends JToolBar implements java.io.Serializable, ActionListener
{
	private static final String[]	ZOOM_LABELS	= {"50%", "75%", "100%", "150%", "200%", "300%", "400%", "800%",
			"1000%", "2000%",					};

	private static final int[]		ZOOM_VALUES	= {50, 75, 100, 150, 200, 300, 400, 800, 1000, 2000,};

	public final static int			OPEN		= 1;

	public final static int			PRINT		= 2;

	public final static int			ZOOM		= 4;

	public final static int			NAVIGATE	= 8;

	public final static int			SAVE		= 16;
	public ViewerToolBar()
	{
		this(OPEN | PRINT | ZOOM | NAVIGATE | SAVE);
	}
	public ViewerToolBar(int flags)
	{
		setFloatable(false);
		setRollover(true);
		setLayout(new FlowLayout(FlowLayout.LEFT));
		if ((flags & ZOOM) != 0)
		{
			m_Zoom = new JComboBox(ZOOM_LABELS);
			m_Zoom.setBorder(null);
			m_Zoom.setSelectedIndex(2); // 100 %
			m_Zoom.setEnabled(false);
			m_Zoom.addItemListener(new ItemListener()
			{
				public void itemStateChanged(ItemEvent ev)
				{
					setZoomPercent(ZOOM_VALUES[m_Zoom.getSelectedIndex()]);
				}
			});
			add(m_Zoom);
		}
		if ((flags & NAVIGATE) != 0)
		{
			m_NavPanel = new JNavigationPanel(new Navigator());
			m_NavPanel.setBorder(null);
			add(m_NavPanel);
		}
		if ((flags & OPEN) != 0)
		{
			m_Open = new JButton(Icons.OPEN);
			m_Open.setActionCommand("Open");
			m_Open.setToolTipText("Open");
			m_Open.setEnabled(false);
			m_Open.addActionListener(this);
			add(m_Open);
		}
		if ((flags & PRINT) != 0)
		{
			m_Print = new JButton(Icons.PRINT);
			m_Print.setActionCommand("Print");
			m_Print.setToolTipText("Print");
			m_Print.addActionListener(this);
			add(m_Print);
		}
	}
	public void setFileFilter(FileFilter ff)
	{
		m_FileFilter = ff;
	}
	public void actionPerformed(ActionEvent ev)
	{
		if (ev.getActionCommand().equals("Open"))
		{
			JFileChooser c = new JFileChooser();
			if (m_FileFilter != null)
			{
				c.setFileFilter(m_FileFilter);
			}
			int retVal = c.showOpenDialog(getFrame());
			if (retVal == JFileChooser.APPROVE_OPTION) open(c.getSelectedFile());
		}
		else if (ev.getActionCommand().equals("Print"))
		{
			IPrinter printer = PrinterUtil.getPrinter();
			printer.print((Printable) m_Bean, new Properties(), getFrame());
		}
	}
	public void open(java.io.File f)
	{
		if (m_Bean != null)
		{
			try
			{
				m_Bean.read(f);
				if (m_NavPanel != null)
				{
					m_NavPanel.refresh();
					m_NavPanel.setStatus();
				}
				if (m_Bean instanceof Zoomable && m_Zoom != null)
				{
					((Zoomable) m_Bean).setZoomPercent(ZOOM_VALUES[m_Zoom.getSelectedIndex()]);
				}
				m_Bean.update();
			}
			catch (java.io.IOException ex)
			{
				JOptionPane.showMessageDialog(getFrame(), "Can't load file: " + ex);
			}
		}
	}
	public void open(Object data)
	{
		if (m_Bean != null)
		{
			m_Bean.setObject(data);
			if (m_NavPanel != null)
			{
				m_NavPanel.refresh();
				m_NavPanel.setStatus();
			}
			if (m_Bean instanceof Zoomable && m_Zoom != null)
			{
				((Zoomable) m_Bean).setZoomPercent(ZOOM_VALUES[m_Zoom.getSelectedIndex()]);
			}
			m_Bean.update();
		}
	}
	public void open(InputStream is)
	{
		if (m_Bean != null)
		{
			try
			{
				m_Bean.read(is);
				if (m_NavPanel != null)
				{
					m_NavPanel.refresh();
					m_NavPanel.setStatus();
				}
				if (m_Bean instanceof Zoomable && m_Zoom != null)
				{
					((Zoomable) m_Bean).setZoomPercent(ZOOM_VALUES[m_Zoom.getSelectedIndex()]);
				}
				m_Bean.update();
			}
			catch (java.io.IOException ex)
			{
				JOptionPane.showMessageDialog(getFrame(), "Can't load file: " + ex);
			}
		}
	}
	public void setViewer(IViewerCanvas bean)
	{
		m_Bean = bean;
		if (m_Zoom != null) m_Zoom.setEnabled(m_Bean instanceof Zoomable);
		if (m_Print != null) m_Print.setEnabled(m_Bean instanceof Printable);
		if (m_Open != null) m_Open.setEnabled(m_Bean != null);
		if (m_NavPanel != null)
		{
			INavigation nav = m_NavPanel.getNavigation();
			//ensure that first page get display
			//issue happened when Glint Canvas is reused. 
			//and the last view is on a x page but new view has lesser page.
			if (nav.getTotalPage()>0)
				nav.first();
			m_NavPanel.refresh();
			m_NavPanel.setStatus();
		}
		if (m_Bean instanceof Zoomable && m_Zoom != null)
		{
			((Zoomable) m_Bean).setZoomPercent(ZOOM_VALUES[m_Zoom.getSelectedIndex()]);
		}
	}
	public IViewerCanvas getViewer()
	{
		return m_Bean;
	}
	private void setZoomPercent(int zoom)
	{
		if (m_Bean instanceof Zoomable)
		{
			((Zoomable) m_Bean).setZoomPercent(zoom);
			m_Bean.update();
		}
	}
	private JFrame getFrame()
	{
		Container c = getParent();
		while (!(c instanceof JFrame))
		{
			c = c.getParent();
			if (c == null) return null;
		}
		return (JFrame) c;
	}

	private IViewerCanvas		m_Bean;

	private JComboBox			m_Zoom;

	private JButton				m_Open;

	private JButton				m_Print;

	private JNavigationPanel	m_NavPanel;

	private FileFilter			m_FileFilter;
	private class Navigator implements INavigation
	{
		public void first()
		{
			if (m_Bean != null) m_Bean.first();
		}
		public boolean previous()
		{
			if (m_Bean != null)
				return m_Bean.previous();
			else return false;
		}
		public boolean next()
		{
			if (m_Bean != null)
				return m_Bean.next();
			else return false;
		}
		public void last()
		{
			if (m_Bean != null) m_Bean.last();
		}
		public boolean gopage(int index)
		{
			if (m_Bean != null)
				return m_Bean.gopage(index);
			else return false;
		}
		public int getTotalPage()
		{
			if (m_Bean != null)
				return m_Bean.getTotalPage();
			else return 0;
		}
		public int getCurrentPage()
		{
			if (m_Bean != null)
				return m_Bean.getCurrentPage();
			else return 0;
		}
	}
}
// end of ViewerToolBar.java
